/*eslint no-var: "error" */
/*eslint no-unused-vars: "off" */
/*eslint-env es6*/
// Put this file in path/to/plugin/amd/src
// You can call it anything you like

define(['jquery', 'block_gradelevel/jquery.animate-shadow', 'core/str', 'core/ajax', 'block_gradelevel/jscolor',
        'block_gradelevel/handlers', 'block_gradelevel/debugger'], function ($, $shadow, str, ajax, jscolor, handlers, Debugger) {

    let debug = new Debugger("leveleditor");
    debug.enable();

    let skill_id = null;
    let $skill_table = null;

    let self = {
        init: function init() {

            // Find skill table and skill id
            $skill_table = $("table#level_config");
            skill_id = Number($skill_table.attr('data-skill'));

            if (!skill_id && skill_id !== 0 && skill_id !== '0') {
                debug.error("Cannot find configured skill id");
                return;
            }

            // make sure all uint type numbers are not able to go below 0
            $(document).on('input', "input[type=number].uint", function (e) {
                let $this = $(this);
                if ($this.val() < 0) {
                    $this.val(0);
                }
            });

            // On click handler for add level link
            $("a[data-action=addlevel]").on('click', function (e) {
                self.add_levelrow(-255, '', '7F7F7F');
            });

            // On click handler for save changes button
            $("button[data-action=savechanges]").on('click', self.submit);
            // On click handler for save changes button
            $("button[data-action=resetlevels]").on('click', self.resetlevels);


        },
        add_levelrow: function (id, points, color) {
            // create new row
            let $newtr = $("<tr data-rowid='"+id+"'>"
                + "<td><input data-name='points' type='number' class='uint' value='"+points+"'></td>"
                + "<td><input type='text' data-name='color' class='jscolor' value='"+color+"'></td></tr>");

            // apply jscolor to new row
            $newtr.find("input.jscolor").each(function () {
                let picker = new jscolor(this);
            });

            // and add to the body of the table
            $skill_table.find("tbody").append($newtr);
        },
        refresh: function refresh() {
            debug.info("Attempting to refresh");

            // perform refresh call
            let promises = ajax.call([{
                methodname: 'block_gradelevel_list_levels',
                args: { skill_id: skill_id },
            }]);

            // and link promise returns to callbacks
            promises[0].done(self.success_refill_table).fail(self.fail_report_exception);
        },
        submit: function submit(evt) {
            let $table = $("table#level_config");

            let level_data = [];
            $table.find("tbody tr[data-rowid]").each(function () {
                let $this = $(this);
                let row = {
                    id: Number($this.attr('data-rowid')),
                    points: $this.find("input[data-name=points]").val(),
                    badgecolor: $this.find("input[data-name=color]").val(),
                };

                if (row.points === "") { row.points = -255; }
                row.points = Number(row.points);

                level_data.push(row);
            });

            //debug.info("Attempting to submit", level_data);

            // perform ajax call
            let promises = ajax.call([{
                methodname: 'block_gradelevel_submit_levels',
                args: { skill_id: skill_id, levels: level_data },
            }]);

            // and link promise returns to callbacks
            promises[0].done(self.success_refill_table).fail(handlers.fail_report_exception);
        },
        resetlevels: function resetlevels(evt) {
            let $table = $("table#level_config");

            // Send a request to delete all rows
            let level_data = [];
            $table.find("tbody tr[data-rowid]").each(function () {
                let $this = $(this);
                let row = {
                    id: Number($this.attr('data-rowid')),
                    points: -255, // means: delete row
                    badgecolor: "", // totally irrelevent when requesting a delete
                };

                level_data.push(row);
            });

            //debug.info("Attempting to submit", level_data);

            // perform ajax call
            let promises = ajax.call([{
                methodname: 'block_gradelevel_submit_levels',
                args: { skill_id: skill_id, levels: level_data },
            }]);

            // and link promise returns to callbacks
            promises[0].done(self.success_refill_table).fail(handlers.fail_report_exception);
        },
        success_refill_table: function success_refill_table(response) {
            //debug.info("Response from webservice: ", response);
            let $tbody = $skill_table.find('tbody');
            $tbody.empty();
            for (let idx in response) {
                let lvl = response[idx];
                //debug.info("Level:", lvl);
                self.add_levelrow(lvl.id, lvl.points, lvl.badgecolor);
            }
            let $inputs = $tbody.find("input[data-name='points']");

            $inputs
                .animate({ boxShadow: '0px 0px 5px 3px #3FDFCF'}, 400)
                .delay(200)
                .animate({ boxShadow: 'none'}, 800);

        },


    };
    return self;
});